@@ -1,10 +1,10 @@
 package Net::DNS::Resolver;
 
 #
-# $Id: Resolver.pm 1068 2012-12-06 10:38:51Z willem $
+# $Id: Resolver.pm 1115 2013-09-20 09:26:13Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1068 $)[1];
+$VERSION = (qw$LastChangedRevision: 1115 $)[1];
 
 =head1 NAME
 
@@ -21,23 +21,19 @@ BEGIN {
 	for ($^O) {				## Perl OS identifier
 
 		/cygwin/ && do {
-			eval { require Net::DNS::Resolver::MSWin32; };
-
-			unless ($@) {
+			if ( eval { require Net::DNS::Resolver::MSWin32; } ) {
 				@ISA = qw(Net::DNS::Resolver::MSWin32);
 				last;
 			}
 		};
 
-
-		eval "require Net::DNS::Resolver::$_;";
-
-		unless ($@) {
+		if ( eval "require Net::DNS::Resolver::$_;" ) {
 			@ISA = ("Net::DNS::Resolver::$_");
-		} else {
-			require Net::DNS::Resolver::UNIX;
-			@ISA = qw(Net::DNS::Resolver::UNIX);
+			last;
 		}
+
+		require Net::DNS::Resolver::UNIX;
+		@ISA = qw(Net::DNS::Resolver::UNIX);
 	}
 }
 
@@ -430,11 +426,19 @@ $res->bgisready >> or C<IO::Select> to find out if the socket is ready
 before reading it.
 
 bgsend does not support persistent sockets.
-bgsend does not support the usevc option (TCP).
+
+B<BEWARE>:
+bgsend does not support the usevc option (TCP) and operates on UDP only;
+Answers may not fit in an UDP packet and might be truncated. Truncated 
+packets will B<not> be retried over TCP automatically and should be handled
+by the caller.
 
 =head2 bgread
 
     $packet = $res->bgread($socket);
+    if ($packet->header->tc) { 
+    	# Retry over TCP (blocking).
+    }
     undef $socket;
 
 Reads the answer from a background query (see L</bgsend>).  The argument
@@ -451,6 +455,9 @@ The programmer should close or destroy the socket object after reading it.
         # do some other processing
     }
     $packet = $res->bgread($socket);
+    if ($packet->header->tc) { 
+    	# Retry over TCP (blocking).
+    }
     $socket = undef;
 
 Determines whether a socket is ready for reading.  The argument is
