@@ -1,6 +1,6 @@
 package Net::DNS::Resolver::Base;
 #
-# $Id: Base.pm 1074 2012-12-10 20:46:01Z willem $
+# $Id: Base.pm 1158 2014-01-10 22:30:40Z willem $
 #
 
 use strict;
@@ -24,7 +24,7 @@ use IO::Select;
 use Net::DNS;
 use Net::DNS::Packet;
 
-$VERSION = (qw$LastChangedRevision: 1074 $)[1];
+$VERSION = (qw$LastChangedRevision: 1158 $)[1];
 
 
 #
@@ -218,7 +218,7 @@ sub read_env {
 
 	if (exists $ENV{'RES_OPTIONS'}) {
 		foreach ($ENV{'RES_OPTIONS'} =~ m/(\S+)/g) {
-			my ($name, $val) = split(m/:/);
+			my ($name, $val) = split(m/:/,$_,2);
 			$val = 1 unless defined $val;
 			$config->{$name} = $val if exists $config->{$name};
 		}
@@ -238,7 +238,7 @@ sub read_config_file {
 
 	local *FILE;
 
-	open(FILE, "<", $file) or return;
+	open(FILE, $file) or return;
 	local $/ = "\n";
 	local $_;
 
@@ -393,7 +393,7 @@ sub nameserver { &nameservers }
 
 sub cname_addr {
 	# TODO 20081217
-	# This code does not follow CNAME chanes, it only looks inside the packet. Out of bailiwick will fail.
+	# This code does not follow CNAME chains, it only looks inside the packet. Out of bailiwick will fail.
 	# Also it is not IP agnostic
 	my $names  = shift;
 	my $packet = shift;
@@ -619,18 +619,21 @@ sub send_tcp {
 			}
 
 			my $ans = Net::DNS::Packet->new(\$buf, $self->{debug});
-			$self->errorstring($@);
+			my $error = $@;
+
+			unless ( defined $ans ) {
+			      $self->errorstring($error);
+			} else {
+				my $rcode = $ans->header->rcode;
+				$self->errorstring( $error || $rcode );
 
-			if (defined $ans) {
 				$ans->answerfrom($self->answerfrom);
 
-				if ($ans->header->rcode ne "NOERROR" &&
-				    $ans->header->rcode ne "NXDOMAIN"){
+				if ( $rcode ne "NOERROR" && $rcode ne "NXDOMAIN" ) {
 					# Remove this one from the stack
-					print "RCODE: ".$ans->header->rcode ."; trying next nameserver\n" if $self->{'debug'};
+					print "RCODE: $rcode; trying next nameserver\n" if $self->{debug};
 					$lastanswer=$ans;
 					next NAMESERVER ;
-
 				}
 
 			}
@@ -882,14 +885,16 @@ sub send_udp {
 				      if $self->{'debug'};
 
 				  my $ans = Net::DNS::Packet->new(\$buf, $self->{debug});
-				  $self->errorstring($@);
+				  my $error = $@;
 
-				  if (defined $ans) {
+				  unless ( defined $ans ) {
+				      $self->errorstring($error);
+				  } else {
 				      my $header = $ans->header;
 				      next SELECTOR unless ( $header->qr || $self->{'ignqrid'});
 				      next SELECTOR unless  ( ($header->id == $packet->header->id) || $self->{'ignqrid'} );
 				      my $rcode = $header->rcode;
-				      $self->errorstring($rcode) unless $@;
+				      $self->errorstring( $error || $rcode);
 
 				      $ans->answerfrom($self->answerfrom);
 				      if ($rcode ne "NOERROR" && $rcode ne "NXDOMAIN"){
